@@ -1,6 +1,199 @@
 # This contains what used to be in CGH.SparseCCA.R and MultiSparseCCA.R
 
-
+#' Perform sparse canonical correlation analysis using the penalized matrix
+#' decomposition.
+#'
+#' Given matrices X and Z, which represent two sets of features on the same set
+#' of samples, find sparse u and v such that u'X'Zv is large.  For X and Z, the
+#' samples are on the rows and the features are on the columns. X and Z must
+#' have same number of rows, but may (and usually will) have different numbers
+#' of columns. The columns of X and/or Z can be unordered or ordered. If
+#' unordered, then a lasso penalty will be used to obtain the corresponding
+#' canonical vector. If ordered, then a fused lasso penalty will be used; this
+#' will result in smoothness.
+#'
+#' This function is useful for performing an integrative analysis of two sets
+#' of measurements taken on the same set of samples: for instance, gene
+#' expression and CGH measurements on the same set of patients. It takes in two
+#' data sets, called x and z, each of which have (the same set of) samples on
+#' the rows. If z is a matrix of CGH data with *ordered* CGH spots on the
+#' columns, then use typez="ordered". If z consists of unordered columns, then
+#' use typez="standard". Similarly for typex.
+#'
+#' This function performs the penalized matrix decomposition on the data matrix
+#' $X'Z$. Therefore, the results should be the same as running the PMD function
+#' on t(x)\%*\% z. However, when ncol(x)>>nrow(x) and ncol(z)>>nrow(z) then
+#' using the CCA function is much faster because it avoids computation of
+#' $X'Z$.
+#'
+#' The CCA criterion is as follows: find unit vectors $u$ and $v$ such that
+#' $u'X'Zv$ is maximized subject to constraints on $u$ and $v$.  If
+#' typex="standard" and typez="standard" then the constraints on $u$ and $v$
+#' are lasso ($L_1$). If typex="ordered" then the constraint on $u$ is a fused
+#' lasso penalty (promoting sparsity and smoothness). Similarly if
+#' typez="ordered".
+#'
+#' When type x is "standard": the L1 bound of u is penaltyx*sqrt(ncol(x)).
+#'
+#' When typex is "ordered": penaltyx controls the amount of sparsity and
+#' smoothness in u, via the fused lasso penalty: $lambda sum_j |u_j| + lambda
+#' sum_j |u_j - u_(j-1)|$. If NULL, then it will be chosen adaptively from the
+#' data.
+#'
+#' @aliases CCA print.CCA
+#' @param x Data matrix; samples are rows and columns are features. Cannot
+#' contain missing values.
+#' @param z Data matrix; samples are rows and columns are features.  Cannot
+#' contain missing values.
+#' @param typex Are the columns of x unordered (type="standard") or ordered
+#' (type="ordered")? If "standard", then a lasso penalty is applied to u, to
+#' enforce sparsity. If "ordered" (generally used for CGH data), then a fused
+#' lasso penalty is applied, to enforce both sparsity and smoothness.
+#' @param typez Are the columns of z unordered (type="standard") or ordered
+#' (type="ordered")? If "standard", then a lasso penalty is applied to v, to
+#' enforce sparsity. If "ordered" (generally used for CGH data), then a fused
+#' lasso penalty is applied, to enforce both sparsity and smoothness.
+#' @param penaltyx The penalty to be applied to the matrix x, i.e. the penalty
+#' that results in the canonical vector u. If typex is "standard" then the L1
+#' bound on u is penaltyx*sqrt(ncol(x)). In this case penaltyx must be between
+#' 0 and 1 (larger L1 bound corresponds to less penalization). If "ordered"
+#' then it's the fused lasso penalty lambda, which must be non-negative (larger
+#' lambda corresponds to more penalization).
+#' @param penaltyz The penalty to be applied to the matrix z, i.e. the penalty
+#' that results in the canonical vector v. If typez is "standard" then the L1
+#' bound on v is penaltyz*sqrt(ncol(z)). In this case penaltyz must be between
+#' 0 and 1 (larger L1 bound corresponds to less penalization). If "ordered"
+#' then it's the fused lasso penalty lambda, which must be non-negative (larger
+#' lambda corresponds to more penalization).
+#' @param K The number of u's and v's desired; that is, the number of canonical
+#' vectors to be obtained.
+#' @param niter How many iterations should be performed? Default is 15.
+#' @param v The first K columns of the v matrix of the SVD of X'Z. If NULL,
+#' then the SVD of X'Z will be computed inside the CCA function. However, if
+#' you plan to run this function multiple times, then save a copy of this
+#' argument so that it does not need to be re-computed (since that process can
+#' be time-consuming if X and Z both have high dimension).
+#' @param trace Print out progress?
+#' @param standardize Should the columns of x and z be centered (to have mean
+#' zero) and scaled (to have standard deviation 1)? Default is TRUE.
+#' @param xnames An optional vector of column names for x.
+#' @param znames An optional vector of column names for z.
+#' @param chromx Used only if typex is "ordered"; allows user to specify a
+#' vector of length ncol(x) giving the chromosomal location of each CGH spot.
+#' This is so that smoothness will be enforced within each chromosome, but not
+#' between chromosomes.
+#' @param chromz Used only if typez is "ordered"; allows user to specify a
+#' vector of length ncol(z) giving the chromosomal location of each CGH spot.
+#' This is so that smoothness will be enforced within each chromosome, but not
+#' between chromosomes.
+#' @param upos If TRUE, then require elements of u to be positive. FALSE by
+#' default. Can only be used if type is "standard".
+#' @param uneg If TRUE, then require elements of u to be negative. FALSE by
+#' default.  Can only be used if type is "standard".
+#' @param vpos If TRUE, require elements of v to be positive. FALSE by default.
+#' Can only be used if type is "standard".
+#' @param vneg If TRUE, require elements of v to be negative. FALSE by default.
+#' Can only be used if type is "standard".
+#' @param outcome If you would like to incorporate a phenotype into CCA
+#' analysis - that is, you wish to find features that are correlated across the
+#' two data sets and also correlated with a phenotype - then use one of
+#' "survival", "multiclass", or "quantitative" to indicate outcome type.
+#' Default is NULL.
+#' @param y If outcome is not NULL, then this is a vector of phenotypes - one
+#' for each row of x and z. If outcome is "survival" then these are survival
+#' times; must be non-negative. If outcome is "multiclass" then these are class
+#' labels (1,2,3,...). Default NULL.
+#' @param cens If outcome is "survival" then these are censoring statuses for
+#' each observation. 1 is complete, 0 is censored. Default NULL.
+#' @return \item{u}{u is output. If you asked for multiple factors then each
+#' column of u is a factor. u has dimension nxK if you asked for K factors.}
+#' \item{v}{v is output. If you asked for multiple factors then each column of
+#' v is a factor. v has dimension pxK if you asked for K factors.} \item{d}{A
+#' vector of length K, which can alternatively be computed as the diagonal of
+#' the matrix $u'X'Zv$.} \item{v.init}{The first K factors of the v matrix of
+#' the SVD of x'z. This is saved in case this function will be re-run later.}
+#' @seealso \link{PMD},\link{CCA.permute}
+#' @references
+#' Witten D. M., Tibshirani R.,  and Hastie, T. (2009)
+#' \emph{A penalized matrix decomposition, with applications to sparse principal components and canonical correlation analysis}, \emph{Biostatistics, Gol 10 (3), 515-534, Jul 2009}\cr
+#' @examples
+#'
+#' # first, do CCA with type="standard"
+#' # A simple simulated example
+#' set.seed(3189)
+#' u <- matrix(c(rep(1,25),rep(0,75)),ncol=1)
+#' v1 <- matrix(c(rep(1,50),rep(0,450)),ncol=1)
+#' v2 <- matrix(c(rep(0,50),rep(1,50),rep(0,900)),ncol=1)
+#' x <- u%*%t(v1) + matrix(rnorm(100*500),ncol=500)
+#' z <- u%*%t(v2) + matrix(rnorm(100*1000),ncol=1000)
+#' # Can run CCA with default settings, and can get e.g. 3 components
+#' out <- CCA(x,z,typex="standard",typez="standard",K=3)
+#' print(out,verbose=TRUE) # To get less output, just print(out)
+#' # Or can use CCA.permute to choose optimal parameter values
+#' perm.out <- CCA.permute(x,z,typex="standard",typez="standard",nperms=7)
+#' print(perm.out)
+#' plot(perm.out)
+#' out <- CCA(x,z,typex="standard",typez="standard",K=1,
+#' 	   penaltyx=perm.out$bestpenaltyx,penaltyz=perm.out$bestpenaltyz,
+#' 	   v=perm.out$v.init)
+#' print(out)
+#'
+#'
+#' ##### The remaining examples are commented out, but uncomment to run: ######
+#'
+#' # Not run, to save time:
+#' \dontrun{
+#' ## Now try CCA with a constraint that elements of u must be negative and
+#' ## elements of v must be positive:
+#' perm.out <- CCA.permute(x,z,typex="standard",typez="standard",nperms=7,
+#' penaltyxs=seq(.1,.7,len=10), penaltyzs=seq(.1,.7,len=10), uneg=TRUE, vpos=TRUE)
+#' print(perm.out)
+#' plot(perm.out)
+#' out <- CCA(x,z,typex="standard",typez="standard",K=1,
+#' 	   penaltyx=perm.out$bestpenaltyx,penaltyz=perm.out$bestpenaltyz,
+#'            v=perm.out$v.init, uneg=TRUE, vpos=TRUE)
+#' print(out)
+#'
+#'
+#' ## Suppose we also have a quantitative outcome, y, and we want to find
+#' ## features in x and z that are correlated with each other and with the
+#' ## outcome:
+#' y <- rnorm(nrow(x))
+#' perm.out <- CCA.permute(x,z,typex="standard",typez="standard",
+#' 			outcome="quantitative",y=y, nperms=6)
+#' print(perm.out)
+#' out<-CCA(x,z,typex="standard",typez="standard",outcome="quantitative",
+#' 	 y=y,penaltyx=perm.out$bestpenaltyx,penaltyz=perm.out$bestpenaltyz)
+#' print(out)
+#'
+#' ## now, do CCA with type="ordered"
+#' ## Example involving the breast cancer data: gene expression + CGH
+#' set.seed(22)
+#' data(breastdata)
+#' attach(breastdata)
+#' dna <- t(dna)
+#' rna <- t(rna)
+#' perm.out <- CCA.permute(x=rna,z=dna[,chrom==1],typex="standard",
+#' 		       	typez="ordered",nperms=5,penaltyxs=seq(.02,.7,len=10))
+#' ## We run CCA using all gene exp. data, but CGH data on chrom 1 only.
+#' print(perm.out)
+#' plot(perm.out)
+#' out <- CCA(x=rna,z=dna[,chrom==1], typex="standard", typez="ordered",
+#' 	   penaltyx=perm.out$bestpenaltyx,
+#'            v=perm.out$v.init, penaltyz=perm.out$bestpenaltyz,
+#'            xnames=substr(genedesc,1,20),
+#'            znames=paste("Pos", sep="", nuc[chrom==1]))
+#' # Save time by inputting  lambda and v
+#' print(out) # could do print(out,verbose=TRUE)
+#' print(genechr[out$u!=0]) # Cool! The genes associated w/ gain or loss
+#' ## on chrom 1 are located on chrom 1!!
+#' par(mfrow=c(1,1))
+#' PlotCGH(out$v, nuc=nuc[chrom==1], chrom=chrom[chrom==1],
+#' main="Regions of gain/loss on Chrom 1 assoc'd with gene expression")
+#' detach(breastdata)
+#' }
+#'
+#' @export CCA
 CCA <- function(x, z, typex=c("standard", "ordered"), typez=c("standard","ordered"), penaltyx=NULL, penaltyz=NULL, K=1, niter=15, v=NULL, trace=TRUE, standardize=TRUE, xnames=NULL, znames=NULL, chromx=NULL, chromz=NULL, upos=FALSE, uneg=FALSE, vpos=FALSE, vneg=FALSE, outcome=NULL, y=NULL, cens=NULL){
   if(ncol(x)<2) stop("Need at least two features in dataset x.")
   if(ncol(z)<2) stop("Need at least two features in dataset z.")
@@ -31,7 +224,7 @@ CCA <- function(x, z, typex=c("standard", "ordered"), typez=c("standard","ordere
   v <- CheckVs(v,x,z,K)
   if(is.null(penaltyx)){
     if(typex=="standard") penaltyx <- .3#pmax(1.001,.3*sqrt(ncol(x)))/sqrt(ncol(x))
-    if(typex=="ordered")  penaltyx <- ChooseLambda1Lambda2(as.numeric(CheckVs(NULL,z,x,1))) # v[,1] used to be NULL 
+    if(typex=="ordered")  penaltyx <- ChooseLambda1Lambda2(as.numeric(CheckVs(NULL,z,x,1))) # v[,1] used to be NULL
   }
   if(is.null(penaltyz)){
     if(typez=="standard") penaltyz <- .3#pmax(1.001,.3*sqrt(ncol(z)))/sqrt(ncol(z))
@@ -71,7 +264,8 @@ CCA <- function(x, z, typex=c("standard", "ordered"), typez=c("standard","ordere
   return(out)
 }
 
-
+#' @method print CCA
+#' @export
 print.CCA <- function(x,verbose=FALSE,...){
   cat("Call: ")
   dput(x$call)
@@ -105,7 +299,7 @@ print.CCA <- function(x,verbose=FALSE,...){
       print(us, quote=FALSE, sep="\t")
       cat(fill=T)
       print(vs, quote=FALSE, sep="\t")
-    }  
+    }
   }
 }
 
@@ -126,7 +320,7 @@ CCAAlgorithm <- function(x,z,v,typex,typez,penaltyx,penaltyz,K,niter,trace,chrom
     if(vpos && sum(abs(v.init[v.init[,k]>0,k]))<sum(abs(v.init[v.init[,k]<0,k]))) v.init[,k] <- -v.init[,k]
     if(vneg && sum(abs(v.init[v.init[,k]<0,k]))<sum(abs(v.init[v.init[,k]>0,k]))) v.init[,k] <- -v.init[,k]
     out <- SparseCCA(xres,zres,v.init[,k],typex,typez,penaltyx, penaltyz,niter,trace, upos, uneg, vpos, vneg,chromx,chromz)
-    coef <- out$d 
+    coef <- out$d
     d <- c(d, coef)
     xres <- rbind(xres, sqrt(coef)*t(out$u))
     zres <- rbind(zres, -sqrt(coef)*t(out$v))
@@ -203,7 +397,7 @@ SparseCCA <- function(x,y,v,typex,typez,penaltyx, penaltyz,niter,trace, upos, un
         su <- soft(argu,lamu)
         u <-  matrix(su/l2n(su), ncol=1)
       }else if(typex=="ordered"){
-        yv <- y%*%v 
+        yv <- y%*%v
         if(is.null(chromx)) chromx <- rep(1, ncol(x))
         for(j in unique(chromx)){
           xyv <- as.numeric(t(yv)%*%x[,chromx==j])#as.numeric(t(x[,chromx==j])%*%yv)
@@ -270,21 +464,22 @@ SparseCCA <- function(x,y,v,typex,typez,penaltyx, penaltyz,niter,trace, upos, un
 }
 
 CheckVs <- function(v,x,z,K){ # If v is NULL, then get v as appropriate.
-  if(!is.null(v) && !is.matrix(v)) v <- matrix(v,nrow=ncol(z))
+    ##print(list(v=v, x = x, z = z, K = K))
+    if(!is.null(v) && !is.matrix(v)) v <- matrix(v,nrow=ncol(z))
   if(!is.null(v) && ncol(v)<K) v <- NULL
   if(!is.null(v) && ncol(v)>K) v <- matrix(v[,1:K],ncol=K)
   if(is.null(v) && ncol(z)>nrow(z) && ncol(x)>nrow(x)){
     v <- try(matrix(fastsvd(x,z)$v[,1:K],ncol=K), silent=TRUE)
     attempt <- 1
-    while(class(v)=="try-error" && attempt < 10){
+    while(("try-error" %in% class(v))  && attempt < 10){
       v <- try(matrix(fastsvd(x,z)$v[,1:K],ncol=K), silent=TRUE)
       attempt <- attempt+1
     }
     if(attempt==10) stop("Problem computing SVD.")
   } else if (is.null(v) && (ncol(z)<=nrow(z) || ncol(x)<=nrow(x))){
     attempt <- 1
-    v <- try(matrix(svd(t(x)%*%z)$v[,1:K],ncol=K), silent=TRUE)    
-    while(class(v)=="try-error" && attempt<10){
+    v <- try(matrix(svd(t(x)%*%z)$v[,1:K],ncol=K), silent=TRUE)
+    while(("try-error" %in% class(v)) && attempt<10){
       v <- try(matrix(svd(t(x)%*%z)$v[,1:K],ncol=K), silent=TRUE)
       attempt <- attempt+1
     }
@@ -312,7 +507,7 @@ CCA.permute.both <- function(x,z,typex,typez,penaltyxs,penaltyzs,niter,v,trace,n
   for(i in 1:nperms){
     if(trace && .Platform$OS.type!="windows") cat("\n Permutation ",i," out of ", nperms, " ")
 #   #  if(trace && .Platform$OS.type=="windows" && i==1) pb <- winProgressBar(title="Doing Permutations", min=0, max=1, initial=(i/nperms))
-#     # if(trace && .Platform$OS.type=="windows" && i>1) setWinProgressBar(pb, value=(i/nperms)) 
+#     # if(trace && .Platform$OS.type=="windows" && i>1) setWinProgressBar(pb, value=(i/nperms))
     sampz <- sample(1:nrow(z))
     sampx <- sample(1:nrow(x))
     for(j in 1:length(penaltyxs)){
@@ -349,7 +544,8 @@ CCA.permute.both <- function(x,z,typex,typez,penaltyxs,penaltyzs,niter,v,trace,n
 }
 
 
-  
+#' @method plot CCA.permute
+#' @export
 plot.CCA.permute <- function(x,...){
   penaltyxs <- x$penaltyxs
   penaltyzs <- x$penaltyzs
@@ -402,7 +598,7 @@ CCA.permute.zonly<- function(x,z,typex,typez,penaltyx,penaltyzs,niter,v,trace,np
   for(i in 1:nperms){
         if(trace && .Platform$OS.type!="windows") cat("\n Permutation ",i," out of ", nperms, " ")
 #     #if(trace && .Platform$OS.type=="windows" && i==1) pb <- winProgressBar(title="Doing Permutations", min=0, max=1, initial=(i/nperms))
-#     #if(trace && .Platform$OS.type=="windows" && i>1) setWinProgressBar(pb, value=(i/nperms)) 
+#     #if(trace && .Platform$OS.type=="windows" && i>1) setWinProgressBar(pb, value=(i/nperms))
     sampz <- sample(1:nrow(z))
     sampx <- sample(1:nrow(x))
     for(j in 1:length(penaltyzs)){
@@ -499,7 +695,7 @@ CCA.permute.xonly<- function(x,z,typex,typez,penaltyxs,penaltyz,niter,v,trace,np
   for(i in 1:nperms){
     if(trace && .Platform$OS.type!="windows") cat("\n Permutation ",i," out of ", nperms, " ")
 #     #if(trace && .Platform$OS.type=="windows" && i==1) pb <- winProgressBar(title="Doing Permutations", min=0, max=1, initial=(i/nperms))
-#     #if(trace && .Platform$OS.type=="windows" && i>1) setWinProgressBar(pb, value=(i/nperms)) 
+#     #if(trace && .Platform$OS.type=="windows" && i>1) setWinProgressBar(pb, value=(i/nperms))
     sampz <- sample(1:nrow(z))
     sampx <- sample(1:nrow(x))
     for(j in 1:length(penaltyxs)){
@@ -537,6 +733,141 @@ CCA.permute.xonly<- function(x,z,typex,typez,penaltyxs,penaltyz,niter,v,trace,np
 
 
 
+
+
+
+
+#' Select tuning parameters for sparse canonical correlation analysis using the
+#' penalized matrix decomposition.
+#'
+#' This function can be used to automatically select tuning parameters for
+#' sparse CCA using the penalized matrix decompostion. For each data set x and
+#' z, two types are possible: (1) type "standard", which does not assume any
+#' ordering of the columns of the data set, and (2) type "ordered", which
+#' assumes that columns of the data set are ordered and thus that corresponding
+#' canonical vector should be both sparse and smooth (e.g. CGH data).
+#'
+#' For X and Z, the samples are on the rows and the features are on the
+#' columns.
+#'
+#' The tuning parameters are selected using a permutation scheme. For each
+#' candidate tuning parameter value, the following is performed: (1) The
+#' samples in X are randomly permuted nperms times, to obtain matrices
+#' $X*_1,X*_2,...$. (2) Sparse CCA is run on each permuted data set $(X*_i,Z)$
+#' to obtain factors $(u*_i, v*_i)$. (3) Sparse CCA is run on the original data
+#' (X,Z) to obtain factors u and v. (4) Compute $c*_i=cor(X*_i u*_i,Z v*_i)$
+#' and $c=cor(Xu,Zv)$. (5) Use Fisher's transformation to convert these
+#' correlations into random variables that are approximately normally
+#' distributed. Let Fisher(c) denote the Fisher transformation of c. (6)
+#' Compute a z-statistic for Fisher(c), using
+#' $(Fisher(c)-mean(Fisher(c*)))/sd(Fisher(c*))$. The larger the z-statistic,
+#' the "better" the corresponding tuning parameter value.
+#'
+#' This function also gives the p-value for each pair of canonical variates
+#' (u,v) resulting from a given tuning parameter value. This p-value is
+#' computed as the fraction of $c*_i$'s that exceed c (using the notation of
+#' the previous paragraph).
+#'
+#' Using this function, only the first left and right canonical variates are
+#' considered in selection of the tuning parameter.
+#'
+#' Note that x and z must have same number of rows. This function
+#' performs just a one-dimensional search in tuning parameter space,
+#' even if penaltyxs and penaltyzs both are vectors: the pairs
+#' `(penaltyxs[1],penaltyzs[1])`,
+#' `(penaltyxs[2],penaltyzs[2])`,.... are considered.
+#' @param x Data matrix; samples are rows and columns are features.
+#' @param z Data matrix; samples are rows and columns are features. Note that x
+#' and z must have the same number of rows, but may (and generally will) have
+#' different numbers of columns.
+#' @param typex Are the columns of x unordered (type="standard") or ordered
+#' (type="ordered")? If "standard", then a lasso penalty is applied to v, to
+#' enforce sparsity. If "ordered" (generally used for CGH data), then a fused
+#' lasso penalty is applied, to enforce both sparsity and smoothness.
+#' @param typez Are the columns of z unordered (type="standard") or ordered
+#' (type="ordered")? If "standard", then a lasso penalty is applied to v, to
+#' enforce sparsity. If "ordered" (generally used for CGH data), then a fused
+#' lasso penalty is applied, to enforce both sparsity and smoothness.
+#' @param penaltyxs The set of x penalties to be considered. If
+#' typex="standard", then the L1 bound on u is penaltyxs*sqrt(ncol(x)). If
+#' "ordered", then it's the lambda for the fused lasso penalty. The user can
+#' specify a single value or a vector of values. If penaltyxs is a vector and
+#' penaltyzs is a vector, then the vectors must have the same length. If NULL,
+#' then the software will automatically choose a single lambda value if type is
+#' "ordered", or a grid of (L1 bounds)/sqrt(ncol(x)) if type is "standard".
+#' @param penaltyzs The set of z penalties to be considered. If
+#' typez="standard", then the L1 bound on v is penaltyzs*sqrt(ncol(z)). If
+#' "ordered", then it's the lambda for the fused lasso penalty. The user can
+#' specify a single value or a vector of values. If penaltyzs is a vector and
+#' penaltyzs is a vector, then the vectors must have the same length. If NULL,
+#' then the software will automatically choose a single lambda value if type is
+#' "ordered", or a grid of (L1 bounds)/sqrt(ncol(z)) if type is "standard".
+#' @param niter How many iterations should be performed each time CCA is
+#' called? Default is 3, since an approximate estimate of u and v is acceptable
+#' in this case, and otherwise this function can be quite time-consuming.
+#' @param v The first K columns of the v matrix of the SVD of X'Z. If NULL,
+#' then the SVD of X'Z will be computed inside this function. However, if you
+#' plan to run this function multiple times, then save a copy of this argument
+#' so that it does not need to be re-computed (since that process can be
+#' time-consuming if X and Z both have high dimension).
+#' @param trace Print out progress?
+#' @param nperms How many times should the data be permuted? Default is 25. A
+#' large value of nperms is very important here, since the formula for
+#' computing the z-statistics requires a standard deviation estimate for the
+#' correlations obtained via permutation, which will not be accurate if nperms
+#' is very small.
+#' @param standardize Should the columns of X and Z be centered (to have mean
+#' zero) and scaled (to have standard deviation 1)? Default is TRUE.
+#' @param chromx Used only if typex="ordered"; a vector of length ncol(x) that
+#' allows you to specify which chromosome each CGH spot is on. If NULL, then it
+#' is assumed that all CGH spots are on same chromosome.
+#' @param chromz Used only if typex="ordered"; a vector of length ncol(z) that
+#' allows you to specify which chromosome each CGH spot is on. If NULL, then it
+#' is assumed that all CGH spots are on same chromosome.
+#' @param upos If TRUE, then require all elements of u to be positive in sign.
+#' Default is FALSE. Can only be used if type is standard.
+#' @param uneg If TRUE, then require all elements of u to be negative in sign.
+#' Default is FALSE. Can only be used if type is standard.
+#' @param vpos If TRUE, then require all elements of v to be positive in sign.
+#' Default is FALSE.  Can only be used if type is standard.
+#' @param vneg If TRUE, then require all elements of v to be negative in sign.
+#' Default is FALSE. Can only be used if type is standard.
+#' @param outcome If you would like to incorporate a phenotype into CCA
+#' analysis - that is, you wish to find features that are correlated across the
+#' two data sets and also correlated with a phenotype - then use one of
+#' "survival", "multiclass", or "quantitative" to indicate outcome type.
+#' Default is NULL.
+#' @param y If outcome is not NULL, then this is a vector of phenotypes - one
+#' for each row of x and z. If outcome is "survival" then these are survival
+#' times; must be non-negative. If outcome is "multiclass" then these are class
+#' labels. Default NULL.
+#' @param cens If outcome is "survival" then these are censoring statuses for
+#' each observation. 1 is complete, 0 is censored. Default NULL.
+#' @return \item{zstat}{The vector of z-statistics, one per element of
+#' sumabss.} \item{pvals}{The vector of p-values, one per element of sumabss.}
+#' \item{bestpenaltyx}{The x penalty that resulted in the highest z-statistic.}
+#' \item{bestpenaltyz}{The z penalty that resulted in the highest z-statistic.}
+#' \item{cors}{The value of cor(Xu,Zv) obtained for each value of sumabss.}
+#' \item{corperms}{The nperms values of cor(X*u*,Zv*) obtained for each value
+#' of sumabss, where X* indicates the X matrix with permuted rows, and u* and
+#' v* are the output of CCA using data (X*,Z).} \item{ft.cors}{The result of
+#' applying Fisher transformation to cors.} \item{ft.corperms}{The result of
+#' applying Fisher transformation to corperms.} \item{nnonzerous}{Number of
+#' non-zero u's resulting from applying CCA to data (X,Z) for each value of
+#' sumabss.} \item{nnonzerouv}{Number of non-zero v's resulting from applying
+#' CCA to data (X,Z) for each value of sumabss.} \item{v.init}{The first factor
+#' of the v matrix of the SVD of x'z. This is saved in case this function (or
+#' the CCA function) will be re-run later.}
+#' @seealso \link{PMD},\link{CCA}
+#'
+#' @references
+#' Witten D. M., Tibshirani R.,  and Hastie, T. (2009)
+#' \emph{A penalized matrix decomposition, with applications to sparse principal components and canonical correlation analysis}, \emph{Biostatistics, Gol 10 (3), 515-534, Jul 2009}\cr
+#' @examples
+#'
+#' # See examples in CCA function
+#'
+#' @export CCA.permute
 CCA.permute <- function(x,z,typex=c("standard", "ordered"), typez=c("standard","ordered"), penaltyxs=NULL, penaltyzs=NULL, niter=3,v=NULL,trace=TRUE,nperms=25, standardize=TRUE, chromx=NULL, chromz=NULL,upos=FALSE, uneg=FALSE, vpos=FALSE, vneg=FALSE, outcome=NULL, y=NULL, cens=NULL){
   if(ncol(x)<2) stop("Need at least 2 features in data set x.")
   if(ncol(z)<2) stop("Need at least 2 features in data set z.")
@@ -574,6 +905,9 @@ CCA.permute <- function(x,z,typex=c("standard", "ordered"), typez=c("standard","
   return(out)
 }
 
+
+#' @method print CCA.permute
+#' @export
 print.CCA.permute <- function(x,...){
   cat("Call: ")
   dput(x$call)
@@ -646,5 +980,5 @@ CCAPhenotypeZeroSome <- function(x,z,y,qt=.8,cens=NULL,outcome=c("quantitative",
   znew <- z
   znew[,!keep.z] <- 0
   return(list(x=xnew,z=znew))
-}      
+}
 
